/*
  Tests for the NanoMotor board :
  The program activates the two digital ports (configured as outputs), where we
  can connect a led or a buzzer. Then reads the two analog ports and shows their values.
  The first analog port (where we can connect a potentiometer) controls the speed
  of the two DC motors, and the angle of the two servos attached.
  The second analog port (with an LDR) shows its value to the serial monitor.
*/

#include <FreeSensors.h>

FSnanoMotor NMT;
FSdigOutput DIG1;
FSdigOutput DIG2;
FSanInput POT;
FSanInput LDR;
FSdcMotor MOT1;
FSdcMotor MOT2;
FSservoMotor SRV1;
FSservoMotor SRV2;
FSserial SER;
int value1;
int value2;
int angle;
int speed;

void setup() {
    NMT.begin();
    DIG1.connect(NMT, DIO1);
    DIG2.connect(NMT, DIO2);
    POT.connect(NMT, AI1);
    LDR.connect(NMT, AI2);
    MOT1.connect(NMT, DC1);
    MOT2.connect(NMT, DC2);
    SRV1.connect(NMT, AO1);
    SRV2.connect(NMT, AO2);
    SER.connect(NMT, SER1);
    // ------------ Alternative syntax
 /* NMT.connect(DIG1, DIO1);
    NMT.connect(DIG2, DIO2);
    NMT.connect(POT, AI1);
    NMT.connect(LDR, AI2);
    NMT.connect(MOT1, DC1);
    NMT.connect(MOT2, DC2);
    NMT.connect(SRV1, AO1);
    NMT.connect(SRV2, AO2);
    NMT.connect(SER, SER1); */
}

void loop() {
    DIG1.writeValue(HIGH);
    DIG2.writeValue(HIGH);
    delay(400);

    value1 = POT.readValue();            // 0 .. 100
    speed = map(value1,0,100,0,255);
    MOT1.forward(speed);
    MOT2.forward(speed);
    angle = map(value1,0,100,0,180);
    SRV1.setAngle(angle);
    SRV2.setAngle(angle);

    value2 = LDR.readValue();            // 0 .. 100
    SER.writeString("AI1="+String(value1)+" AI2="+String(value2));

    DIG1.writeValue(LOW);
    DIG2.writeValue(LOW);
    delay(400);
}
