/*
  FSaccelGyro.h - FreeSensors library for Arduino
  Copyright (C) 2018 Mario A. Guerra. All rights reserved.
  License: GNU GPL v3 http://www.gnu.org/licenses/gpl.html

  Accelerometer-Gyroscope for FreeSensors:
  based on the MPU6050_tockn library: easy communicating with the MPU6050
  https://github.com/Tockn/MPU6050_tockn  version=1.4.0
*/

#ifndef _FSaccelGyro_h
#define _FSaccelGyro_h

#include <Wire.h>
#include "FSi2c.h"

class FSaccelGyro : public FSi2c {
    private:
        float accCoef, gyroCoef;
        short rangeAccel, rangeGyro;
        int16_t rawAccX, rawAccY, rawAccZ, rawTemp;
        int16_t rawGyroX, rawGyroY, rawGyroZ;
        float accelXoffset, accelYoffset, accelZoffset;
        float gyroXoffset, gyroYoffset, gyroZoffset;
        float accX, accY, accZ;                     // Acceleration with values in G units
        float speedX, speedY, speedZ;               // Linear velocity with values in m/sec units
        float gyroX, gyroY, gyroZ;                  // Angular velocity with values in degree/sec units
        float angleAccX, angleAccY;                 // Angles in degrees acording to accelerometer
        float angleGyroX, angleGyroY, angleGyroZ;   // Angles in degrees acording to gyroscope
        float angleX, angleY, angleZ;               // Angles in degrees using complementary filter
        unsigned long prevTime;
        void readRawValues();
    public:
        FSaccelGyro();
        void begin();
        void initialize();
        void updateValues();
        void setCoefficients(float accC, float gyrC);
        void setRanges(short accR, short gyrR);
        void setAccelOffsets(float x, float y, float z);
        void setGyroOffsets(float x, float y, float z);
        void setZeroPosition();                             // Set initial position for angles and speed
        void calibrate(bool console = false);
        float getAccelX() { return accX; };                 // Acceleration in [G]
        float getAccelY() { return accY; };
        float getAccelZ() { return accZ; };
        float getSpeedX() { return speedX; };               // Linear velocity [m/s]
        float getSpeedY() { return speedY; };
        float getSpeedZ() { return speedZ; };
        float getGyroX() { return gyroX; };                 // Angular velocity [°/s]
        float getGyroY() { return gyroY; };
        float getGyroZ() { return gyroZ; };
        float getAccelAngleX() { return angleAccX; };       // Angle [°]
        float getAccelAngleY() { return angleAccY; };
        float getGyroAngleX() { return angleGyroX; };       // Angle [°]
        float getGyroAngleY() { return angleGyroY; };
        float getGyroAngleZ() { return angleGyroZ; };
        float getAngleX() { return angleX; };               // Angle [°]
        float getAngleY() { return angleY; };
        float getAngleZ() { return angleZ; };
};

#endif
