/*
  FSaiport.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSaiport.h"

FSaiport::FSaiport() {
    addrPins[0] = 13; // LSB
    addrPins[1] = 12;
    addrPins[2] = 11; // MSB
    analogPin = A3;
}

void FSaiport::begin() {
    port[0].jack = 1;    port[0].type = fptAI;
    port[1].jack = 2;    port[1].type = fptAI;
    port[2].jack = 3;    port[2].type = fptAI;
    port[3].jack = 4;    port[3].type = fptAI;
    port[4].jack = 5;    port[4].type = fptAI;
    port[5].jack = 6;    port[5].type = fptAI;
    port[6].jack = 7;    port[6].type = fptAI;
    port[7].jack = 8;    port[7].type = fptAI;
    port[8].jack = 1;    port[8].type = fptSER;
    port[9].jack = 1;    port[9].type = fptI2C;
    pinModeFS(addrPins[0],OUTPUT);
    pinModeFS(addrPins[1],OUTPUT);
    pinModeFS(addrPins[2],OUTPUT);
}

// Use this function only if board jumpers are changed 
void FSaiport::setAnalogPin(byte ap) {
    analogPin = ap;
}

int FSaiport::readAnalogValue(byte jack) {
    if ((jack<1) || (jack>8)) return 0;
    jack = jack - 1;
    // The last 3 bits of pin are written as address
    digitalWriteFS(addrPins[0],bitRead(jack,0)); // LSB
    digitalWriteFS(addrPins[1],bitRead(jack,1));
    digitalWriteFS(addrPins[2],bitRead(jack,2)); // MSB
    delay(2);  // CMOS 4051 address propagation time is 450 ns
    return analogReadFS(analogPin);
}

// DEPRECATED functions

int FSaiport::analogRead(byte pin) {
    return readAnalogValue(pin);
}
