/*
  FSbuttonsAI.cpp - FreeSensors library for Arduino
  Copyright (C) 2018 Mario A. Guerra. All rights reserved.
  License: GNU GPL v3 http://www.gnu.org/licenses/gpl.html
*/

#include "FSbuttonsAI.h"

/* Values for reading six buttons from an Analog Input.
   The following values are in the middle between the expected
   readings of button values (in the range 0..1023)
*/
#define halfStep1  120   // should be 85, put in 120 for better noise rejection
#define halfStep3  256
#define halfStep5  426
#define halfStep7  597
#define halfStep9  767
#define halfStep11 938

FSbuttonsAI::FSbuttonsAI() {
    mtype = fmtAI;
    jack = 0;
    button = fsbNONE;
}

void FSbuttonsAI::begin() {
    jack = getPortJack();
}

byte FSbuttonsAI::decodeButtonsAI(uint16_t adcValue) {
         if (adcValue>halfStep11) return fsbGO;
    else if (adcValue>halfStep9)  return fsbFORWARD;
    else if (adcValue>halfStep7)  return fsbBACKWARD;
    else if (adcValue>halfStep5)  return fsbRIGHT;
    else if (adcValue>halfStep3)  return fsbLEFT;
    else if (adcValue>halfStep1)  return fsbPAUSE;
    else return fsbNONE;
}

/* Performs three secuential readings in ~ 40 ms,
   implements a software debouncing
*/
byte FSbuttonsAI::buttonPressed() {
    uint16_t adc;                             // raw analog value read
    byte caso0,caso1,caso2;

    toShield->readAnalogValue(jack);          // switch the ADC channel and hold for 104 µs
    delay(1);
    adc = toShield->readAnalogValue(jack);    // proper first readout
    if (adc<halfStep1) {
        button = fsbNONE;
        return button;
    }
    caso0 = decodeButtonsAI(adc);
    delay(18);
    adc = toShield->readAnalogValue(jack);    // second readout
    caso1 = decodeButtonsAI(adc);
    delay(20);
    adc = toShield->readAnalogValue(jack);    // third readout
    caso2 = decodeButtonsAI(adc);
    if (caso0==caso1 && caso1==caso2) {
        button = caso0;
    } else {
        button = fsbNONE;                     // if the 3 are not the same, suppose no button was pressed
    }
    return button;
}

/* Returns the string name of the last button pressed.
   Needs buttonPressed() method called before.
 */
String FSbuttonsAI::buttonName() {
    return buttonName(button);
}

String FSbuttonsAI::buttonName(byte xbutton) {
    switch (xbutton) {
        case fsbGO :       return "GO";
        case fsbFORWARD :  return "FORWARD";
        case fsbBACKWARD : return "BACKWARD";
        case fsbRIGHT :    return "RIGHT";
        case fsbLEFT :     return "LEFT";
        case fsbPAUSE :    return "PAUSE";
        default :          return "NONE";
    }
}
