/*
  FSdcMotor.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSdcMotor.h"

FSdcMotor::FSdcMotor() {
    mtype = fmtMOT;
    reversed = false;
}

// The following methods works after a successfull connection
// to a shield port of type fptDC
void FSdcMotor::begin() {
}

void FSdcMotor::stop() {
    toShield->dcMotor00(getPortJack());
    toShield->disableDC(getPortJack());
}

void FSdcMotor::forward() {
    toShield->enableDC(getPortJack());
    if (!reversed) {
        toShield->dcMotorCW(getPortJack());
    } else {
        toShield->dcMotorCCW(getPortJack());
    }
}

void FSdcMotor::backward() {
    toShield->enableDC(getPortJack());
    if (!reversed) {
        toShield->dcMotorCCW(getPortJack());
    } else {
        toShield->dcMotorCW(getPortJack());
    }
}

// parameter speed is in range 0..255
void FSdcMotor::forward(byte speed) {
    if (speed==0) {
        stop();
        return;
    }
    forward();
    if (speed==0xFF) return;
    // If speed is lower than full, tries to control it
    toShield->setSpeedDC(getPortJack(), speed);
}

// parameter speed is in range 0..255
void FSdcMotor::backward(byte speed) {
    if (speed==0) {
        stop();
        return;
    }
    backward();
    if (speed==0xFF) return;
    // If speed is lower than full, tries to control it
    toShield->setSpeedDC(getPortJack(), speed);
}

void FSdcMotor::setReversed(boolean rev /* = true */) {
    reversed = rev;
}

// Converts speed in percentage 0..100 to byte in range 0..255
byte FSdcMotor::percentToSpeed(int speed) {
    return lround(speed*255.0/100);
}
