/*
  FSdioport.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSdioport.h"

FSdioport::FSdioport() {
    stype = fstDIOPORT;
    direccion = 0x00;  // all OUTPUTs
}

void FSdioport::begin() {
    begin(PCFA_010);
}

void FSdioport::begin(byte address) {
    port[0].jack = 1;    port[0].type = fptDIOF;
    port[1].jack = 2;    port[1].type = fptDIOF;
    port[2].jack = 3;    port[2].type = fptDIOF;
    port[3].jack = 4;    port[3].type = fptDIOF;
    port[4].jack = 5;    port[4].type = fptDIOF;
    port[5].jack = 6;    port[5].type = fptDIOF;
    port[6].jack = 7;    port[6].type = fptDIOF;
    port[7].jack = 8;    port[7].type = fptDIOF;
    port[8].jack = 1;    port[8].type = fptSER;
    port[9].jack = 1;    port[9].type = fptI2C;
    attach(address);
    registro = 0;   // start from a known state, with all bits off
    writeValue();
}

// This connect is needed to differentiate between FSshield  and FSmodule connects
bool FSdioport::connect(FSmodule &xmod, int xjack) {
    return FSshield::connect(xmod, xjack);
}

/* DIOport shield needs to match the pin definition with jumper position for that pin.
   mode is defined in Arduino as: INPUT 0x0, OUTPUT 0x1, INPUT_PULLUP 0x2,
   but inside the function is changed to INPUT=1, OUTPUT=0
   On inputs there is allways an implicit pullup when we write 1 to that pins.
   DIOport shield does not support Ultrasonic (because the need of dynamic change between input/output).
*/
void FSdioport::setPinMode(byte jack, byte mode) {
    switch (mode) {
        case OUTPUT: bitWrite(direccion, jack-1, 0); break;
        case INPUT:
        case INPUT_PULLUP: bitWrite(direccion, jack-1, 1); break;
    }
}

bool FSdioport::readDigitalValue(byte jack) {
    if ((jack<1) || (jack>8)) return 0;
    readValue();
    return bitRead(registro, jack-1);
}

void FSdioport::writeDigitalValue(byte jack, byte value) {
    if ((jack<1) || (jack>8)) return;
    // On input pins always write 1
    registro = registro | direccion;
    bitWrite(registro, jack-1, value);
    writeValue();
}

// DEPRECATED functions

void FSdioport::pinMode(byte pin, byte mode) {
    setPinMode(pin, mode);
}

void FSdioport::digitalWrite(byte pin, byte value) {
    writeDigitalValue(pin, value);
}

byte FSdioport::digitalRead(byte pin) {
    return readDigitalValue(pin);
}
