/*
  FSdirect.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSdirect.h"

FSdirect::FSdirect() {
    stype = fstDIRECT;
}

void FSdirect::begin() {
    port[0].jack = 1;    port[0].type = fptDIO;    port[0].pin = 3;
    port[1].jack = 2;    port[1].type = fptDIO;    port[1].pin = 4;
    port[2].jack = 3;    port[2].type = fptDIO;    port[2].pin = 7;
    port[3].jack = 4;    port[3].type = fptDIO;    port[3].pin = 8;
    port[4].jack = 1;    port[4].type = fptAI;     port[4].pin = 0;
    port[5].jack = 2;    port[5].type = fptAI;     port[5].pin = 1;
    port[6].jack = 3;    port[6].type = fptAI;     port[6].pin = 2;
    port[7].jack = 4;    port[7].type = fptAI;     port[7].pin = 3;
    port[8].jack = 1;    port[8].type = fptSER;
    port[9].jack = 1;    port[9].type = fptI2C;
}

// Use this function only if board jumpers are changed
void FSdirect::setDigitalPins(byte p1, byte p2, byte p3, byte p4) {
    port[0].pin = p1;
    port[1].pin = p2;
    port[2].pin = p3;
    port[3].pin = p4;
}

// Use this function only if board jumpers are changed 
void FSdirect::setAnalogPins(byte a1, byte a2, byte a3, byte a4) {
    port[4].pin = a1;
    port[5].pin = a2;
    port[6].pin = a3;
    port[7].pin = a4;
}

// mode is defined in Arduino as: INPUT 0x0, OUTPUT 0x1, INPUT_PULLUP 0x2
void FSdirect::setPinMode(byte jack, byte mode) {
    switch (mode) {
        case OUTPUT:
        case INPUT:
        case INPUT_PULLUP: pinModeFS(port[jack-1].pin, mode); break;
        case ULTRASONIC: connect(sonar[jack-1],jack); break;
    }
}

bool FSdirect::readDigitalValue(byte jack) {
    return digitalReadFS(port[jack-1].pin);
}

void FSdirect::writeDigitalValue(byte jack, byte value) {
    digitalWriteFS(port[jack-1].pin, value);
}

int FSdirect::readAnalogValue(byte jack) {
    return analogReadFS(port[jack+3].pin);
}

// DEPRECATED functions

void FSdirect::pinMode(byte pin, byte mode) {
    setPinMode(pin, mode);
}

void FSdirect::digitalWrite(byte pin, byte value) {
    writeDigitalValue(pin, value);
}

byte FSdirect::digitalRead(byte pin) {
    return readDigitalValue(pin);
}

int FSdirect::analogRead(byte pin) {
    return readAnalogValue(pin);
}

int FSdirect::ultrasonicDistance(byte pin) {
    int dist;
    dist = sonar[pin-1].pingCM();
    // when there is no echo until max distance, return this max distance plus 2 cm
    if (dist==0) dist = sonar[pin-1].getMaxDistance() + 2;
    return dist;
}

int FSdirect::getMaxUltrasonicDistance(byte pin) {
    return sonar[pin-1].getMaxDistance();
}

void FSdirect::setMaxUltrasonicDistance(byte pin, int maxd) {
    sonar[pin-1].setMaxDistance(maxd);
}
