/*
  FShBridge.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FShBridge.h"

#define ENABLE_DCM1 3      // bit in I2C register
#define ENABLE_DCM2 4      // bit in I2C register
#define DCM1_PLUS 5        // bit in I2C register
#define DCM1_MINU 6        // bit in I2C register
#define DCM2_PLUS 7        // bit in I2C register
#define DCM2_MINU 8        // bit in I2C register

FShBridge::FShBridge() {
    stype = fstHBRIDGE;
}

void FShBridge::begin() {
    begin(PCFA_010);
}

void FShBridge::begin(byte address) {
    port[0].jack = 1;    port[0].type = fptDC;
    port[1].jack = 2;    port[1].type = fptDC;
    attach(address);
    registro = 0;
    writeValue();
}

// This connect is needed to differentiate between FSshield  and FSmodule connects
bool FShBridge::connect(FSmodule &xmod, int xjack) {
    return FSshield::connect(xmod, xjack);
}

void FShBridge::enableDC(byte jack) {
    if (jack==1) {
        write1bit(ENABLE_DCM1,1);
    } else if (jack==2) {
        write1bit(ENABLE_DCM2,1);
    }
}

void FShBridge::disableDC(byte jack) {
    if (jack==1) {
        write1bit(ENABLE_DCM1,0);
    } else if (jack==2) {
        write1bit(ENABLE_DCM2,0);
    }
}

void FShBridge::setSpeedDC(byte jack, byte value) {
    // may be it could be possible to send a PWM like signal
    // via the I2C, using the bits 3 and 4
}

void FShBridge::dcMotor00(byte jack) {
    if (jack==1) {
        write2bit(DCM1_PLUS,0,DCM1_MINU,0);
    } else if (jack==2) {
        write2bit(DCM2_PLUS,0,DCM2_MINU,0);
    }
}

void FShBridge::dcMotor11(byte jack) {
    if (jack==1) {
        write2bit(DCM1_PLUS,1,DCM1_MINU,1);
    } else if (jack==2) {
        write2bit(DCM2_PLUS,1,DCM2_MINU,1);
    }
}

void FShBridge::dcMotorCW(byte jack) {
    if (jack==1) {
        write2bit(DCM1_PLUS,1,DCM1_MINU,0);
    } else if (jack==2) {
        write2bit(DCM2_PLUS,1,DCM2_MINU,0);
    }
}

void FShBridge::dcMotorCCW(byte jack) {
    if (jack==1) {
        write2bit(DCM1_PLUS,0,DCM1_MINU,1);
    } else if (jack==2) {
        write2bit(DCM2_PLUS,0,DCM2_MINU,1);
    }
}
