/*
  FSi2c.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include <Wire.h>
#include "FSi2c.h"

FSi2c::FSi2c() {
    defaultAddr = 0;   // 0 is an invalid address
    deviceAddr = 0;    // 0 is an invalid address
    registro = 0;
    // jack = 0;       // jack is unique and always the same
}

void FSi2c::begin() {
 /* This method is defined so it could be possible to make instances
    for generic I2C modules, but there is no default address, so before
    using the other methods, it should connect to I2C bus using the
    attach method with proper bus address. */
}

void FSi2c::attach(byte address) {
    deviceAddr = address;
    Wire.begin();
}

bool FSi2c::attached() {
    uint8_t error;
    delay(100);
    Wire.beginTransmission(deviceAddr);
    error = Wire.endTransmission();
    return (!error);
}

byte FSi2c::getAddress() {
    return deviceAddr;
}

void FSi2c::setAddress(byte address) {
    deviceAddr = address;
}

byte FSi2c::getValue() {
    return registro;
}

void FSi2c::setValue(byte value) {
    registro = value;
}

byte FSi2c::readValue() {
    Wire.requestFrom(deviceAddr, (byte)1);
    if (Wire.available()) {
        registro = Wire.read();
    } else {
        // if not available, registro holds its previous value
    }
    return registro;
}

void FSi2c::writeValue() {
    Wire.beginTransmission(deviceAddr);
    Wire.write(registro);
    Wire.endTransmission();
}

void FSi2c::writeValue(byte value) {
    registro = value;
    writeValue();
}

void FSi2c::writeConfig(byte reg, byte data) {
    DBG_PRINT("Writing to register ");
    DBG_PRINT(reg,BIN);
    DBG_PRINT(" data = ");
    DBG_PRINTLN(data,BIN);
    Wire.beginTransmission(deviceAddr);
    Wire.write(reg);
    Wire.write(data);
    Wire.endTransmission();
}
