/*
  FSmodule.h - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#ifndef _FSmodule_h
#define _FSmodule_h

#include <Arduino.h>
#include "FSport.h"
#include "FSshield.h"

#define DEBUG 0         // Set DEBUG to 1 for debug output
#if DEBUG
#   define DBG_PRINT(...)   Serial.print(__VA_ARGS__)
#   define DBG_PRINTLN(...) Serial.println(__VA_ARGS__)
#else
#   define DBG_PRINT(...)
#   define DBG_PRINTLN(...)
#endif

enum fsModuleType {
    fmtUNDEF = 0,
    fmtDO    = 1,    // to gray port (sensors only Output)
    fmtDI    = 2,    // to gray port (sensors only Input)
    fmtDIO   = 3,    // to gray port (sensors that need dynamic change between I and O)
    fmtAI    = 4,    // to blue port
    fmtAO    = 5,    // to brown port
    fmtSRV   = 6,    // to brown port
    fmtSTP   = 7,    // to brown port
    fmtMOT   = 8,    // to red port
    fmtSER   = 9,    // to yellow port
    fmtI2C   = 10,   // to green port
    fmtSPI   = 11    // to white port
};

class FSshield;      // due to circular includes

class FSmodule {
    protected:
        FSshield* toShield;
        FSport* toPort;
        fsModuleType mtype;
    public:
        FSmodule();
        virtual void begin() = 0;
        bool connect(FSshield &xshi, int xjack);
        void setShieldConn(FSshield* pshi);
        void setPortConn(FSport* pport);
        fsModuleType getModuleType();
        FSshield* getShieldConn();
        FSport* getPortConn();
        byte getPortJack();
        byte getPortPin();
        bool isConnected();
};

#endif
