/*
  FSmotor.h - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#ifndef _FSmotor_h
#define _FSmotor_h

#include "FSshield.h"
#include "FSexpanderDIO.h"
#include "FSdcMotor.h"
#include "FSservoMotor.h"
#include <Servo.h>

class FSmotor : public FSshield, public FSexpanderDIO {
    private:
        byte direccion;
        Servo srvs[2];
    public:
        FSmotor();
        void begin();
        void begin(byte address);
        bool connect(FSmodule &xmod, int xjack);
        [[deprecated]] FSdcMotor m1;
        [[deprecated]] FSdcMotor m2;
        // FSdioCapable interface
        void setPinMode(byte jack, byte mode);
        bool readDigitalValue(byte jack);
        void writeDigitalValue(byte jack, byte value);
        // FSaoCapable interface
        void writePwmValue(byte jack, byte value);
        void setPinOutput(byte jack);
        void doPulse(byte jack, byte duration);
        void startServo(byte jack);
        void endServo(byte jack);
        void setServoMinMax(byte jack, int pulseMin, int pulseMax);
        void setServoAngle(byte jack, int angle);
        void setServoPulse(byte jack, int pulse);
        // FShbCapable interface
        void enableDC(byte jack);
        void disableDC(byte jack);
        void setSpeedDC(byte jack, byte value);
        void dcMotor00(byte jack);
        void dcMotor11(byte jack);
        void dcMotorCW(byte jack);
        void dcMotorCCW(byte jack);
        #include "nonFSaiCapable.inc"
        // Following functions only for compatibility with vers < 1.2.0
        [[deprecated]] void pinMode(byte pin, byte mode);
        [[deprecated]] void digitalWrite(byte pin, byte value);
        [[deprecated]] byte digitalRead(byte pin);
};

#endif
