/*
  FSnanoBrown.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSnanoBrown.h"

FSnanoBrown::FSnanoBrown() {
    stype = fstNANOBROWN;
}

void FSnanoBrown::begin() {
    port[0].jack = 1;    port[0].type = fptAI;     port[0].pin = 0;
    port[1].jack = 2;    port[1].type = fptAI;     port[1].pin = 1;
    port[2].jack = 1;    port[2].type = fptDIO;    port[2].pin = 3;
    port[3].jack = 2;    port[3].type = fptDIO;    port[3].pin = 4;
    port[4].jack = 1;    port[4].type = fptAO;     port[4].pin = 5;
    port[5].jack = 2;    port[5].type = fptAO;     port[5].pin = 6;
    port[6].jack = 3;    port[6].type = fptAO;     port[6].pin = 9;
    port[7].jack = 4;    port[7].type = fptAO;     port[7].pin = 10;
    port[8].jack = 1;    port[8].type = fptSER;
    port[9].jack = 1;    port[9].type = fptI2C;
    port[10].jack = 1;   port[10].type = fptSPI;   port[10].pin = 8; // SS pin
}

// mode is defined in Arduino as: INPUT 0x0, OUTPUT 0x1, INPUT_PULLUP 0x2
void FSnanoBrown::setPinMode(byte jack, byte mode) {
    pinModeFS(port[jack+1].pin, mode);
}

bool FSnanoBrown::readDigitalValue(byte jack) {
    return digitalReadFS(port[jack+1].pin);
}

void FSnanoBrown::writeDigitalValue(byte jack, byte value) {
    digitalWriteFS(port[jack+1].pin, value);
}

int FSnanoBrown::readAnalogValue(byte jack) {
    return analogReadFS(port[jack-1].pin);
}

void FSnanoBrown::writePwmValue(byte jack, byte value) {
    analogWriteFS(port[jack+3].pin, value);
}

void FSnanoBrown::setPinOutput(byte jack) {
    pinModeFS(port[jack+3].pin, OUTPUT);
}

void FSnanoBrown::doPulse(byte jack, byte duration) {
    digitalWriteFS(port[jack+3].pin, HIGH);
    delayMicroseconds(duration);
    digitalWriteFS(port[jack+3].pin, LOW);
}

void FSnanoBrown::startServo(byte jack) {
    // Using the "Servo" library initializes an array of 12 servos structs,
    // the constructor of the Servo object sets the index of this array
    srvs[jack-1].attach(port[jack+3].pin, SERVO_PULSE_MIN, SERVO_PULSE_MAX);
}

void FSnanoBrown::endServo(byte jack) {
    srvs[jack-1].detach();
}

void FSnanoBrown::setServoMinMax(byte jack, int pulseMin, int pulseMax) {
    srvs[jack-1].attach(port[jack+3].pin, pulseMin, pulseMax);
}

void FSnanoBrown::setServoAngle(byte jack, int angle) {
    srvs[jack-1].write(angle);
}

void FSnanoBrown::setServoPulse(byte jack, int pulse) {
    srvs[jack-1].writeMicroseconds(pulse);
}
