/*
  FSnanoMotor.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSnanoMotor.h"

#define ENABLE_DCM1 5      // Arduino digital pin
#define ENABLE_DCM2 6      // Arduino digital pin
#define DCM1_PLUS 7        // Arduino digital pin
#define DCM1_MINU 8        // Arduino digital pin
#define DCM2_PLUS 11       // Arduino digital pin
#define DCM2_MINU 12       // Arduino digital pin

FSnanoMotor::FSnanoMotor() {
    stype = fstNANOMOTOR;
}

/* In NanoMotor boards :
     another DIO pins : 16 (analog A2 as digital), 17 (analog A3 as digital)
     another AI pins : A6 (only analog input), A7 (only analog input)
 */
void FSnanoMotor::begin() {
    port[0].jack = 1;    port[0].type = fptAI;     port[0].pin = 0;
    port[1].jack = 2;    port[1].type = fptAI;     port[1].pin = 1;
    port[2].jack = 1;    port[2].type = fptDIO;    port[2].pin = 3;
    port[3].jack = 2;    port[3].type = fptDIO;    port[3].pin = 4;
    port[4].jack = 1;    port[4].type = fptDC;
    port[5].jack = 2;    port[5].type = fptDC;
    port[6].jack = 1;    port[6].type = fptAO;     port[6].pin = 9;
    port[7].jack = 2;    port[7].type = fptAO;     port[7].pin = 10;
    port[8].jack = 1;    port[8].type = fptSER;
    port[9].jack = 1;    port[9].type = fptI2C;
    pinModeFS(ENABLE_DCM1, OUTPUT);  // PWM may control speed
    pinModeFS(ENABLE_DCM2, OUTPUT);  // PWM may control speed
    pinModeFS(DCM1_PLUS, OUTPUT);    // M1+
    pinModeFS(DCM1_MINU, OUTPUT);    // M1-
    pinModeFS(DCM2_PLUS, OUTPUT);    // M2+
    pinModeFS(DCM2_MINU, OUTPUT);    // M2-
}

// mode is defined in Arduino as: INPUT 0x0, OUTPUT 0x1, INPUT_PULLUP 0x2
void FSnanoMotor::setPinMode(byte jack, byte mode) {
    pinModeFS(port[jack+1].pin, mode);
}

bool FSnanoMotor::readDigitalValue(byte jack) {
    return digitalReadFS(port[jack+1].pin);
}

void FSnanoMotor::writeDigitalValue(byte jack, byte value) {
    digitalWriteFS(port[jack+1].pin, value);
}

int FSnanoMotor::readAnalogValue(byte jack) {
    return analogReadFS(port[jack-1].pin);
}

void FSnanoMotor::writePwmValue(byte jack, byte value) {
    analogWriteFS(port[jack+5].pin, value);
}

void FSnanoMotor::setPinOutput(byte jack) {
    pinModeFS(port[jack+5].pin, OUTPUT);
}

void FSnanoMotor::doPulse(byte jack, byte duration) {
    digitalWriteFS(port[jack+5].pin, HIGH);
    delayMicroseconds(duration);
    digitalWriteFS(port[jack+5].pin, LOW);
}

void FSnanoMotor::startServo(byte jack) {
    // Using the "Servo" library initializes an array of 12 servos structs,
    // the constructor of the Servo object sets the index of this array
    srvs[jack-1].attach(port[jack+5].pin, SERVO_PULSE_MIN, SERVO_PULSE_MAX);
}

void FSnanoMotor::endServo(byte jack) {
    srvs[jack-1].detach();
}

void FSnanoMotor::setServoMinMax(byte jack, int pulseMin, int pulseMax) {
    srvs[jack-1].attach(port[jack+5].pin, pulseMin, pulseMax);
}

void FSnanoMotor::setServoAngle(byte jack, int angle) {
    srvs[jack-1].write(angle);
}

void FSnanoMotor::setServoPulse(byte jack, int pulse) {
    srvs[jack-1].writeMicroseconds(pulse);
}

void FSnanoMotor::enableDC(byte jack) {
    if (jack==1) {
        digitalWriteFS(ENABLE_DCM1,1);
    } else if (jack==2) {
        digitalWriteFS(ENABLE_DCM2,1);
    }
}

void FSnanoMotor::disableDC(byte jack) {
    if (jack==1) {
        digitalWriteFS(ENABLE_DCM1,0);
    } else if (jack==2) {
        digitalWriteFS(ENABLE_DCM2,0);
    }
}

void FSnanoMotor::setSpeedDC(byte jack, byte value) {
    if (jack==1) {
        analogWriteFS(ENABLE_DCM1, value);
    } else if (jack==2) {
        analogWriteFS(ENABLE_DCM2, value);
    }
}

void FSnanoMotor::dcMotor00(byte jack) {
    if (jack==1) {
        digitalWriteFS(DCM1_PLUS,LOW);     // M1+
        digitalWriteFS(DCM1_MINU,LOW);     // M1-
    } else if (jack==2) {
        digitalWriteFS(DCM2_PLUS,LOW);     // M2+
        digitalWriteFS(DCM2_MINU,LOW);     // M2-
    }
}

void FSnanoMotor::dcMotor11(byte jack) {
    if (jack==1) {
        digitalWriteFS(DCM1_PLUS,HIGH);    // M1+
        digitalWriteFS(DCM1_MINU,HIGH);    // M1-
    } else if (jack==2) {
        digitalWriteFS(DCM2_PLUS,HIGH);    // M2+
        digitalWriteFS(DCM2_MINU,HIGH);    // M2-
    }
}

void FSnanoMotor::dcMotorCW(byte jack) {
    if (jack==1) {
        digitalWriteFS(DCM1_PLUS,HIGH);    // M1+
        digitalWriteFS(DCM1_MINU,LOW);     // M1-
    } else if (jack==2) {
        digitalWriteFS(DCM2_PLUS,HIGH);    // M2+
        digitalWriteFS(DCM2_MINU,LOW);     // M2-
    }
}

void FSnanoMotor::dcMotorCCW(byte jack) {
    if (jack==1) {
        digitalWriteFS(DCM1_PLUS,LOW);     // M1+
        digitalWriteFS(DCM1_MINU,HIGH);    // M1-
    } else if (jack==2) {
        digitalWriteFS(DCM2_PLUS,LOW);     // M2+
        digitalWriteFS(DCM2_MINU,HIGH);    // M2-
    }
}
