/*
  FSservoMotor.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSservoMotor.h"

FSservoMotor::FSservoMotor() {
    mtype = fmtSRV;
    jack = 0;
    reversed = false;
}

void FSservoMotor::begin() {
    jack = getPortJack();
    toShield->setPinOutput(jack); // redundant, servo attach should call pinMode OUTPUT
    toShield->startServo(jack);   // servos starts at 90° (position of 1500 µs pulse)
}

void FSservoMotor::end() {
    toShield->endServo(jack);
}

void FSservoMotor::setMinMax(int pulseMin, int pulseMax) {
    toShield->setServoMinMax(jack, pulseMin, pulseMax);
}

// The default TowerPro SG90 goes CCW with longer pulses,
// so the angle is reversed by default
void FSservoMotor::setAngle(float angle) {
    int ang = lround(angle);
    if (ang>=0 && ang<=180) {
        ang = (!reversed)? 180-ang:ang;
        toShield->setServoAngle(jack, ang);
    }
}

// The Servo library trims the pulse width between min and max, so if a value
// outside this range is needed, first call setMinMax() with a wider range
void FSservoMotor::setPulse(int pulse) {
    toShield->setServoPulse(jack, pulse);
}

void FSservoMotor::setReversed(boolean rev /* = true */) {
    reversed = rev;
}
