/*
  FSshield.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSshield.h"

FSshield::FSshield() {
    stype = fstUNDEF;
    // When creating the array of ports, they should be all initialized
    // with connected = false and type = fptUNDEF (in their constructor).
    // Later in the proper shield.begin() the actual ports gets their type.
}

bool FSshield::connect(FSmodule &xmod, int xjack) {
    DBG_PRINTLN("Inside FSshield::connect ... ");
    bool compatible;
    bool success = false;
    int mtype = xmod.getModuleType();
    for (int i=0; i<MAX_SHIELD_PORTS; i++) {
        switch (port[i].type) {
            case fptDIO  : compatible = (mtype==fmtDO || mtype==fmtDI || mtype==fmtDIO);
                           break;
            case fptDIOF : compatible = (mtype==fmtDO || mtype==fmtDI);
                           break;
            case fptAI   : compatible = (mtype==fmtAI);
                           break;
            case fptAO   : compatible = (mtype==fmtSRV || mtype==fmtSTP || mtype==fmtAO);
                           break;
            case fptDC   : compatible = (mtype==fmtMOT);
                           break;
            case fptSER  : compatible = (mtype==fmtSER);
                           break;
            case fptI2C  : compatible = (mtype==fmtI2C);
                           break;
            case fptSPI  : compatible = (mtype==fmtSPI);
                           break;
                 default : compatible = false;
        }
        if (compatible && port[i].jack==xjack) {
         /* The connected port property is used in the module isConnected() function.
            Here, it could be possible to verify if the port is already connected and refuse to
            connect, but in some cases more than one module can be connected to the same jack,
            either in programs or physically (for example with Digital Outputs or I2C)
            */
            port[i].connected = true;
            xmod.setShieldConn(this);
            xmod.setPortConn(&port[i]);
            // begin comes after connecting, so the module can know to which shield is connected
            xmod.begin();
            success = true;
            DBG_PRINT("success in connecting jack ");
            DBG_PRINTLN(xjack);
            break;
        }
    }
    return success;
}

fsShieldType FSshield::getShieldType() {
    return stype;
}

void FSshield::pinModeFS(byte pin, byte mode) {
    pinMode(pin, mode);                                  // Arduino native
}

void FSshield::digitalWriteFS(byte pin, byte value) {
    digitalWrite(pin, value);                            // Arduino native
}

byte FSshield::digitalReadFS(byte pin) {
    return digitalRead(pin);                             // Arduino native
}

void FSshield::analogWriteFS(byte pin, int value) {
    analogWrite(pin, value);                             // Arduino native
}

int FSshield::analogReadFS(byte pin) {
    return analogRead(pin);                              // Arduino native
}
