/*
  FSshield.h - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#ifndef _FSshield_h
#define _FSshield_h

#include <Arduino.h>
#include "FSport.h"
#include "FSmodule.h"

#define MAX_SHIELD_PORTS 10
// In the standard <Servo.h> the minimun pulse is 544 µs and the max pulse is 2400 µs,
// giving a center of 1472 µs. Here is optimized for TowerPro SG90 with center at 1500 µs.
#define SERVO_PULSE_MIN 500
#define SERVO_PULSE_MAX 2500

enum fsShieldType {
    fstUNDEF       = 0,
    fstDIRECT      = 1,
    fstDIOPORT     = 2,
    fstAIPORT      = 3,
    fstMOTOR       = 4,
    fstSERVOPORT   = 5,
    fstSPIPORT     = 6,
    fstSERIALDUINO = 7,
    fstNANOMOTOR   = 8,
    fstNANOBROWN   = 9,
    fstHBRIDGE     = 10
};

class FSmodule;         // due to circular includes

class FSshield {
    protected:
        FSport port[MAX_SHIELD_PORTS];
        fsShieldType stype;
        void pinModeFS(byte pin, byte mode);
        void digitalWriteFS(byte pin, byte value);
        byte digitalReadFS(byte pin);
        void analogWriteFS(byte pin, int value);
        int analogReadFS(byte pin);
    public:
        FSshield();
        virtual void begin() = 0;
        bool connect(FSmodule &xmod, int xjack);
        fsShieldType getShieldType();
        // FSdioCapable interface
        virtual void setPinMode(byte jack, byte mode) = 0;
        virtual bool readDigitalValue(byte jack) = 0;
        virtual void writeDigitalValue(byte jack, byte value) = 0;
        // FSaiCapable interface
        virtual int readAnalogValue(byte jack) = 0;
        // FSaoCapable interface
        virtual void writePwmValue(byte jack, byte value) = 0;
        virtual void setPinOutput(byte jack) = 0;
        virtual void doPulse(byte jack, byte duration) = 0;
        virtual void startServo(byte jack) = 0;
        virtual void endServo(byte jack) = 0;
        virtual void setServoMinMax(byte jack, int pulseMin, int pulseMax) = 0;
        virtual void setServoAngle(byte jack, int angle) = 0;
        virtual void setServoPulse(byte jack, int pulse) = 0;
        // FShbCapable interface (H-Bridge for DC motors)
        virtual void enableDC(byte jack) = 0;
        virtual void disableDC(byte jack) = 0;
        virtual void setSpeedDC(byte jack, byte value) = 0;
        virtual void dcMotor00(byte jack) = 0;
        virtual void dcMotor11(byte jack) = 0;
        virtual void dcMotorCW(byte jack) = 0;
        virtual void dcMotorCCW(byte jack) = 0;
};

#endif
