/*
  FSspi.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include <Arduino.h>
#include "FSspi.h"

FSspi::FSspi() {
    mtype = fmtSPI;
    jack = 0;
    pinSS = 0;
}

void FSspi::begin() {
    jack = getPortJack();
    pinSS = getPortPin();
    // The SPI library supports only master mode, and the pinSS must be managed externally
    pinMode(pinSS, OUTPUT);
    digitalWrite(pinSS, HIGH);  // redundant, it should be done in the following begin
    SPI.begin();
    config = SPISettings(SPI_CLOCK_DIV4, MSBFIRST, SPI_MODE0);
}

byte FSspi::readValue(byte reg) {
    byte valor;
    SPI.beginTransaction(config);
    digitalWrite(pinSS, LOW);
    SPI.transfer(reg);
    valor = SPI.transfer(0x00);
    digitalWrite(pinSS, HIGH);
    SPI.endTransaction();
    return valor;
}

void FSspi::readValues(byte reg, byte *values, byte count) {
    if (count == 0) return;
    byte index = 0;
    SPI.beginTransaction(config);
    digitalWrite(pinSS, LOW);
    count--;                                // because the last reading outside the loop
    SPI.transfer(reg);
    while (index < count) {
        values[index] = SPI.transfer(reg);  // read bytes and tell we want to read next
        index++;
    }
    values[index] = SPI.transfer(0x00);     // read the final byte and send 0 to stop reading
    digitalWrite(pinSS, HIGH);
    SPI.endTransaction();
}

void FSspi::writeValue(byte reg, byte value) {
    SPI.beginTransaction(config);
    digitalWrite(pinSS, LOW);
    SPI.transfer(reg);
    SPI.transfer(value);
    digitalWrite(pinSS, HIGH);
    SPI.endTransaction();
}

void FSspi::writeValues(byte reg, byte *values, byte count) {
    if (count == 0) return;
    byte index;
    SPI.beginTransaction(config);
    digitalWrite(pinSS, LOW);
    SPI.transfer(reg);
    for (index = 0; index < count; index++) {
        SPI.transfer(values[index]);
    }
    digitalWrite(pinSS, HIGH);
    SPI.endTransaction();
}
