/*
  FSstepMotor.cpp - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#include "FSstepMotor.h"

FSstepMotor::FSstepMotor() {
    mtype = fmtSTP;
    jack = 0;
    setSpeed(10.0);       // RPM
    prevTime = 0;
    direccion = 0;
    reversed = false;
}

void FSstepMotor::begin() {
    jack = getPortJack();
    toShield->setPinOutput(jack);
    stop();
}

void FSstepMotor::stop() {
    toShield->doPulse(jack, 170);
}

void FSstepMotor::forward() {
    byte durac = (!reversed)? 110:140;
    toShield->doPulse(jack, durac);
}

void FSstepMotor::backward() {
    byte durac = (!reversed)? 140:110;
    toShield->doPulse(jack, durac);
}

// This function needs a call to run() in loop()
// parameter speed is in RPM
void FSstepMotor::forward(float speed) {
    setSpeed(speed);
    direccion = (!reversed)? 1 : 2;
}

// This function needs a call to run() in loop()
// parameter speed is in RPM
void FSstepMotor::backward(float speed) {
    setSpeed(speed);
    direccion = (!reversed)? 2 : 1;
}

void FSstepMotor::run() {
    currTime = micros();
    byte durac = (direccion==1)? 50:80;
    if ((currTime - prevTime) > (stepDelay-durac-70)) {
        if (direccion>0) toShield->doPulse(jack, durac);
        prevTime = currTime;
    }
}

void FSstepMotor::moveStepCW() {
    toShield->doPulse(jack, 50);
}

void FSstepMotor::moveStepCCW() {
    toShield->doPulse(jack, 80);
}

void FSstepMotor::moveStepForward() {
    byte durac = (!reversed)? 50:80;
    toShield->doPulse(jack, durac);
}

void FSstepMotor::moveStepBackward() {
    byte durac = (!reversed)? 80:50;
    toShield->doPulse(jack, durac);
}

void FSstepMotor::moveSteps(long numSteps) {
    long pasos = (!reversed)? numSteps : -1*numSteps;
    byte durac = (pasos>0)? 50:80;
    pasos = abs(pasos);
    for (long i = 0; i < pasos; i++) {
        toShield->doPulse(jack, durac);
        delayMicroseconds(stepDelay-durac-50);
    }
}

void FSstepMotor::moveTurns(float numTurns) {
    long pasosTot = lround(numTurns*stepsPerRev);
    moveSteps(pasosTot);
}

void FSstepMotor::moveAngle(float angle) {
    long pasosTot = lround(angle*stepsPerRev/360.0);
    moveSteps(pasosTot);
}

// parameter speed is in RPM
void FSstepMotor::setSpeed(float speed) {
    velocidad = speed;
    if (velocidad>MAX_STP_RPM) velocidad = MAX_STP_RPM;
    else if (velocidad<MIN_STP_RPM) velocidad = MIN_STP_RPM;
    stepDelay = lround(15100.0/velocidad);
}

void FSstepMotor::setReversed(boolean rev /* = true */) {
    reversed = rev;
}

// Converts speed in percentage 0..100 to RPM
float FSstepMotor::percentToSpeed(int speed) {
    return 1.0*speed*(MAX_STP_RPM - MIN_STP_RPM)/100 + MIN_STP_RPM;
}
