/*
  FSstepMotor.h - FreeSensors library for Arduino.
  Copyright (C) 2019 Mario A. Guerra. All rights reserved.
  This file is part of FreeSensors, licensed under GNU GPL version 3.
*/

#ifndef _FSstepMotor_h
#define _FSstepMotor_h

#include "FSmodule.h"

#define stepsPerRev 4088.0
#define MIN_STP_RPM 1.0
#define MAX_STP_RPM 16.8

class FSstepMotor : public FSmodule {
    private:
        byte jack;
        float velocidad;
        int stepDelay;
        unsigned long currTime;
        unsigned long prevTime;
        byte direccion;
        boolean reversed;
    public:
        FSstepMotor();
        void begin();
        void stop();
        void forward();                            // non-blocking, fixed speed ~ 10 RPM
        void backward();                           // non-blocking, fixed speed ~ 10 RPM
        void forward(float speed);                 // non-blocking, but needs call to run() in loop()
        void backward(float speed);                // non-blocking, but needs call to run() in loop()
        void run();
        void moveStepCW();                         // one step
        void moveStepCCW();                        // one step
        void moveStepForward();                    // one step accounting for reversed
        void moveStepBackward();                   // one step accounting for reversed
        void moveSteps(long numSteps);             // blocking, relative steps (+ CW, - CCW)
        void moveTurns(float numTurns);            // blocking, relative turns (+ CW, - CCW)
        void moveAngle(float angle);               // blocking, relative angle (+ CW, - CCW)
        void setSpeed(float speed);                // speed in RPM for blocking moves
        void setReversed(boolean rev = true);      // normal is CW in front view
        float percentToSpeed(int speed);           // convert percent speed 0..100 to RPM
};

#endif
