/*
  FSultrasonic.h - FreeSensors library for Arduino
  Copyright (C) 2018 Mario A. Guerra. All rights reserved.
  License: GNU GPL v3 http://www.gnu.org/licenses/gpl.html

  UltrasonicFS for FreeSensors (with "one pin" operation)
  is based on NewPing Library - vers 1.8 - 07/30/2016
  Created by Tim Eckel - teckel@leethost.com
  Copyright 2016 License: GNU GPL v3 http://www.gnu.org/licenses/gpl.html
  Project home: https://bitbucket.org/teckel12/arduino-new-ping/wiki/Home
  Blog: http://arduino.cc/forum/index.php/topic,106043.0.html
*/

#ifndef _FSultrasonic_h
#define _FSultrasonic_h

#include "FSmodule.h"
#include <avr/io.h>
#include <avr/interrupt.h>

#define MAX_ULTRA_DISTANCE 200  // Maximum sensor distance defined as 200 cm, no reason to wait for ping longer than sound takes to travel this distance and back
#define US_ROUNDTRIP_CM 57      // Microseconds (µS) it takes sound to travel round-trip 1cm (2cm total), uses integer to save compiled code space. Default=57
#define NO_ECHO 0               // Value returned if there's no ping echo within the specified MAX_ULTRA_DISTANCE
#define MAX_SENSOR_DELAY 5800   // Maximum uS it takes for sensor to start the ping. Default=5800
#define ECHO_TIMER_FREQ 24      // Frequency to check for a ping echo (every 24uS is about 0.4cm accuracy). Default=24
#define PING_MEDIAN_DELAY 29000 // Microsecond delay between pings in the ping_median method. Default=29000
#define PING_OVERHEAD 5         // Ping overhead in microseconds (uS). Default=5

class FSultrasonic : public FSmodule {
    private:
        uint8_t _triggerBit;
        uint8_t _echoBit;
        volatile uint8_t *_triggerOutput;
        volatile uint8_t *_echoInput;
        volatile uint8_t *_triggerMode;
        unsigned int _maxEchoTime;
        unsigned long _max_time;
        boolean ping_trigger();
        unsigned int ping();        // returns time in µsec
    public:
        FSultrasonic();
        void begin();
        void initialize(unsigned int maxDistanceCM);
        unsigned long pingCM();     // returns distance in cm or NO_ECHO
        unsigned long pingMedian(uint8_t it = 5);
        void setMaxDistance(unsigned int maxDistanceCM);
        int getMaxDistance();
        int getDistance();          // returns distance in cm or (maxDistance + 1)
};

#endif
